anova.semipar<-function(num.knots=3,n.iter = 5000, n.burnin = 1000,
         n.thin = 100)

{
##################################################################################################
##################################################################################################
#
#CALL: 
#
#sim.par<-anova.semipar(num.knots=3,n.iter = 5000, n.burnin = 1000,n.thin = 100)
#
##################################################################################################
##################################################################################################
##################################################################################################
##################################################################################################
#
#TECHNICAL DETAILS: 
#
#
#This function 
#
#
##################################################################################################
##################################################################################################
# 
#Requires the package R2WinBUGS (Andrew Gelman)
#
##################################################################################################
##################################################################################################
#
#
#
##################################################################################################
##################################################################################################
# 
#INPUT: 
#	 
#
#
##################################################################################################
##################################################################################################
##################################################################################################
##################################################################################################
# 
#OUPUT: 
#
#	Returns (niter-n.burnin)/n.thin simulations from the posterior distribution of the 
#		model parameters
#
#
##################################################################################################
##################################################################################################

library(R2WinBUGS)

#Read and attach data
file.name="dogdata.txt"

data<-read.table(file=file.name,header=TRUE)
attach(data)

#Obtain the number of observations
n<-length(covariate)
nsubjects<-36
ngroups<-4

#Define the matrix of fixed effects. This matrix could be more complex
#to incorporate other covariates. In this case the WinBUGS program should also be changed
X<-cbind(rep(1,n),covariate)

#Define knots used for the p-spline at equal spaced quantiles of the covariate
num.knots=3
knots<-quantile(unique(covariate), seq(0,1,length=(num.knots+2))[-c(1,(num.knots+2))])

#Obtain the design matrix for random coefficients using a radial basis
Z_K<-(abs(outer(covariate,knots,"-")))^3
OMEGA_all<-(abs(outer(knots,knots,"-")))^3
svd.OMEGA_all<-svd(OMEGA_all)
sqrt.OMEGA_all<-t(svd.OMEGA_all$v %*% (t(svd.OMEGA_all$u)*sqrt(svd.OMEGA_all$d)))
Z<-t(solve(sqrt.OMEGA_all,t(Z_K)))

data<-list("response","X","Z","n","num.knots","nsubjects","ngroups","dog","group")



##################################################################################################
##################################################################################################
#
#
#Define initial values, parameters to be monitored and run WinBUGS
#
##################################################################################################
##################################################################################################
	
	
	inits.delta<-rep(0,72)
	dim(inits.delta)=c(36,2)
	
	inits.d<-rep(0,108)
	dim(inits.d)=c(36,3)

	inits.gamma<-rep(0,8)
	dim(inits.gamma)<-c(4,2)

	inits.c<-rep(0,12)
	dim(inits.c)<-c(4,3)
	
	inits<-function(){list(beta=c(0,0),b=c(0,0,0),taueps=0.01,taub=0.01,
	tauc=0.01,taud=0.01,taudelta=c(0.1,0.1),delta=inits.delta,d=inits.d,gamma=inits.gamma,c=inits.c)}
	parameters<-list("beta","b","gamma","c","delta","d","taub","tauc","taud","groupmean")

	Bayes.fit<- bugs(data, inits, parameters, model.file = "SP_ANOVA.txt",
         	n.chains = 1, n.iter = n.iter, n.burnin = n.burnin,
         	n.thin = n.thin,debug = FALSE, DIC = FALSE, digits = 5, codaPkg = FALSE,
        	bugs.directory = "c:/Program Files/WinBUGS14/")

	attach.bugs(Bayes.fit)




#Obtain the median and 95% credible interval for the overall mean function
Bayes.mean.inference<-rep(0,3*ncol(groupmean))
dim(Bayes.mean.inference)<-c(ncol(groupmean),3)

for (i in 1:ncol(groupmean))

	{
	Bayes.mean.inference[i,]<-quantile(groupmean[,i],probs=c(0.025,0.5,0.975))
	} 

#Start plotting the dog data
#Obtain the posterior simulation from the group mean functions
groupindex=c(0,sum(group<=1),sum(group<=2),sum(group<=3))+1
dogindex=c(9,19,27,36)

par(mfrow=c(2,2))

plot(seq(1,13,2),Bayes.mean.inference[groupindex[1]:(groupindex[1]+6),2],"l",col="red",lwd=2,xlab="Covariate",ylab="Mean function",
ylim=c(min(response),max(response)))
lines(seq(1,13,2),Bayes.mean.inference[groupindex[1]:(groupindex[1]+6),1],lty=20,col="magenta",lwd=2)
lines(seq(1,13,2),Bayes.mean.inference[groupindex[1]:(groupindex[1]+6),3],lty=20,col="magenta",lwd=2)
for (i in 1:dogindex[1]){
	lines(seq(1,13,2),response[(1+(i-1)*7):(7*i)],col="blue",lwd=2)}
title(main="GROUP 1")


plot(seq(1,13,2),Bayes.mean.inference[groupindex[2]:(groupindex[2]+6),2],"l",col="red",lwd=2,xlab="Covariate",ylab="Mean function",
ylim=c(min(response),max(response)))
lines(seq(1,13,2),Bayes.mean.inference[groupindex[2]:(groupindex[2]+6),1],lty=20,col="magenta",lwd=2)
lines(seq(1,13,2),Bayes.mean.inference[groupindex[2]:(groupindex[2]+6),3],lty=20,col="magenta",lwd=2)
for (i in (dogindex[1]+1):dogindex[2]){
	lines(seq(1,13,2),response[(1+(i-1)*7):(7*i)],col="blue",lwd=2)}
title(main="GROUP 2")

plot(seq(1,13,2),Bayes.mean.inference[groupindex[3]:(groupindex[3]+6),2],"l",col="red",lwd=2,xlab="Covariate",ylab="Mean function",
ylim=c(min(response),max(response)))
lines(seq(1,13,2),Bayes.mean.inference[groupindex[3]:(groupindex[3]+6),1],lty=20,col="magenta",lwd=2)
lines(seq(1,13,2),Bayes.mean.inference[groupindex[3]:(groupindex[3]+6),3],lty=20,col="magenta",lwd=2)
for (i in (dogindex[2]+1):dogindex[3]){
	lines(seq(1,13,2),response[(1+(i-1)*7):(7*i)],col="blue",lwd=2)}
title(main="GROUP 3")

plot(seq(1,13,2),Bayes.mean.inference[groupindex[4]:(groupindex[4]+6),2],"l",col="red",lwd=2,xlab="Covariate",ylab="Mean function",
ylim=c(min(response),max(response)))
lines(seq(1,13,2),Bayes.mean.inference[groupindex[4]:(groupindex[4]+6),1],lty=20,col="magenta",lwd=2)
lines(seq(1,13,2),Bayes.mean.inference[groupindex[4]:(groupindex[4]+6),3],lty=20,col="magenta",lwd=2)
for (i in (dogindex[3]+1):dogindex[4]){
	lines(seq(1,13,2),response[(1+(i-1)*7):(7*i)],col="blue",lwd=2)}
title(main="GROUP 4")

return(parameters)
}